# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/10_util.ipynb.

# %% auto 0
__all__ = ['todunder', 'getcard', 'iscard', 'is1char', 'dropext', 'rstr', 'pluralize']

# %% ../nbs/10_util.ipynb 4
from numbers import Number
from typing import Optional
import os, pathlib

# %% ../nbs/10_util.ipynb 5
from .nstr import DOT, NIL, SPACE, SUNDER, DUNDER
from .filt import prepart
from .chck import isdunder

# %% ../nbs/10_util.ipynb 6
def todunder(s: str) -> str:
    return f'__{s}__' if not isdunder(s) else s

# %% ../nbs/10_util.ipynb 7
def getcard(s: str) -> int:
    '''Returns the cardinality of the set of characters for the string.'''
    return len(set(s))

def iscard(s: str, n: int = 1) -> bool:
    '''Checks if a string is composed of a set of `n` characters.'''
    return getcard(s) == n

def is1char(s: str) -> bool:
    '''Checks if a string is composed of a single unique character.'''
    return iscard(s, 1)

# %% ../nbs/10_util.ipynb 8
def dropext(file: str, ext: Optional[str] = None) -> str:
    '''
    Remove the file extension from a file path string or a pathlib.Path object.

    Parameters
    ----------
    file : str or pathlib.Path
        The full path or filename from which the extension should be removed.
    ext : Optional[str], default=None
        Specific extension to remove. If not provided, any extension will be removed.

    Returns
    -------
    str
        File path or filename without the specified extension.

    Notes
    -----
    If the `ext` parameter is not provided, the function will remove the last extension (after the last dot).
    '''
    file = os.fspath(file) if isinstance(file, pathlib.Path) else file
    file = os.path.basename(file)   
    file = prepart(file, DOT) if ext is None else file.replace(ext, NIL)
    return os.path.join(os.path.dirname(file), file)

# %% ../nbs/10_util.ipynb 9
def rstr(n: Number, digits: int = 3) -> str:
    '''Convert a number to a string representation with specified significant digits.

    Parameters
    ----------
    n : Number
        Number to be converted to string.
    digits : int, default=3
        Number of significant digits.

    Returns
    -------
    str
        String representation of the number with the specified significant digits.

    Notes
    -----
    The conversion can also be achieved using the string format mini-language:
    `r = f'{n:.{digits}f}'`.
    '''
    r = str(round(n, digits))
    i, _, d = r.partition(DOT)
    if set(d) != set('0'): return str(r)
    return i if _ == '' else f'{i}.0'

# %% ../nbs/10_util.ipynb 10
def pluralize(word: str, num: float) -> str:
    '''Convert a word to its plural form based on a numeric value.

    Parameters
    ----------
    word : str
        Word to potentially pluralize.
    num : float
        Numeric value to determine whether to pluralize.

    Returns
    -------
    str
        Original word or its plural form depending on the numeric value.

    Notes
    -----
    Returns the word in its original form if the number is 1. Otherwise, converts to plural.
    '''
    if num != 1:
        if word.endswith('y'): return word[:-1] + 'ies'
        else: return word + 's'
    return word
